"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

from ambari_commons.constants import AMBARI_SUDO_BINARY
from resource_management.libraries.functions.default import default
from resource_management.libraries.functions.format import format
from resource_management.libraries.functions.is_empty import is_empty
from resource_management.libraries.script.script import Script
from resource_management.libraries.functions import conf_select
from resource_management.libraries.functions import Direction
from resource_management.libraries.functions import stack_select
from resource_management.libraries.resources import HdfsResource
from resource_management.libraries.functions import StackFeature
from resource_management.libraries.functions.stack_features import check_stack_feature
from resource_management.libraries.functions.stack_features import get_stack_feature_version
from resource_management.libraries.functions.get_stack_version import get_stack_version

import os
import status_params

def get_port_from_url(address):
  if not is_empty(address):
    return address.split(':')[-1]
  else:
    return address

# config object that holds the configurations declared in the -site.xml file
config = Script.get_config()
tmp_dir = Script.get_tmp_dir()
stack_root = Script.get_stack_root()
stack_name = default("/clusterLevelParams/stack_name", None)
retryAble = default("/commandParams/command_retry_enabled", False)
version = default("/commandParams/version", None)
cluster_name = config["clusterName"]

current_version = default("/ambariLevelParams/current_version", None)
upgrade_direction = default("/commandParams/upgrade_direction", None)
stack_version = default("/commandParams/version", None)
sudo = AMBARI_SUDO_BINARY
security_enabled = status_params.security_enabled
fs_root = config['configurations']['core-site']['fs.defaultFS']
solr_conf = "/etc/cloudera-hdp-solr/conf"

solr_port = status_params.solr_port
solr_piddir = status_params.solr_piddir
solr_pidfile = status_params.solr_pidfile

user_group = config['configurations']['cluster-env']['user_group']
fetch_nonlocal_groups = config['configurations']['cluster-env']["fetch_nonlocal_groups"]

# shared configs
java64_home = config['ambariLevelParams']['java_home']
zookeeper_hosts_list = config['clusterHostInfo']['zookeeper_server_hosts']
zookeeper_hosts_list.sort()
# get comma separated list of zookeeper hosts from clusterHostInfo
zookeeper_hosts = ",".join(zookeeper_hosts_list)

#####################################
# Solr configs
#####################################

# Only supporting SolrCloud mode - so hardcode those options
solr_cloudmode = 'true'
solr_dir = '/usr/cloudera-hdp-solr/current/cloudera-hdp-solr/solr'
solr_client_dir = '/usr/hdp/current/solr-client'
solr_bindir = solr_dir + '/bin'
cloud_scripts = solr_dir + '/server/scripts/cloud-scripts'

limits_conf_dir = "/etc/security/limits.d"
solr_user_nofile_limit = default("/configurations/solr-env/solr_user_nofile_limit", "128000")
solr_user_nproc_limit = default("/configurations/solr-env/solr_user_nproc_limit", "65536")

if "solr-env" in config['configurations']:
  solr_hosts = config['clusterHostInfo']['solr_hosts']
  solr_znode = default('/configurations/solr-env/solr_znode', '/solr')
  solr_heap_size = default('/configurations/solr-env/solr_heap_size', 1024)
  solr_heap_pretouch = default('/configurations/solr-env/solr_heap_pretouch', False)
  solr_instance_count = len(config['clusterHostInfo']['solr_hosts'])
  solr_datadir = default('/configurations/solr-env/solr_datadir', '/opt/solr/data')
  solr_data_resources_dir = os.path.join(solr_datadir, 'resources')
  solr_jmx_port = default('/configurations/solr-env/solr_jmx_port', 18983)
  solr_ssl_enabled = default('configurations/solr-env/solr_ssl_enabled', False)
  solr_keystore_location = config['configurations']['solr-env']['solr_keystore_location']
  solr_keystore_password = config['configurations']['solr-env']['solr_keystore_password']
  solr_keystore_type = config['configurations']['solr-env']['solr_keystore_type']
  solr_truststore_location = config['configurations']['solr-env']['solr_truststore_location']
  solr_truststore_password = config['configurations']['solr-env']['solr_truststore_password']
  solr_truststore_type = config['configurations']['solr-env']['solr_truststore_type']
  solr_user = config['configurations']['solr-env']['solr_user']
  solr_log_dir = config['configurations']['solr-env']['solr_log_dir']
  solr_log = format("{solr_log_dir}/solr-install.log")
  solr_env_content = config['configurations']['solr-env']['content']
  solr_hdfs_home_dir = config['configurations']['solr-env']['solr_hdfs_home_dir']
  solr_hdfs_user_dir = format('/user/{solr_user}')
  solr_hdfs_blockcache_enabled = default('/configurations/solr-env/solr_hdfs_blockcache_enabled', False)
  solr_hdfs_blockcache_slab_count = default('/configurations/solr-env/solr_hdfs_blockcache_slab_count', 1)
  solr_hdfs_blockcache_size = default('/configurations/solr-env/solr_hdfs_blockcache_size', 1024)

if upgrade_direction is not None and upgrade_direction == Direction.UPGRADE:
  old_lib_dir=default("/configurations/solr-env/solr_lib_dir", None)

zookeeper_port = default('/configurations/zoo.cfg/clientPort', None)
# get comma separated list of zookeeper hosts from clusterHostInfo
index = 0
zookeeper_quorum = ""
for host in config['clusterHostInfo']['zookeeper_server_hosts']:
  zookeeper_quorum += host + ":" + str(zookeeper_port)
  index += 1
  if index < len(config['clusterHostInfo']['zookeeper_server_hosts']):
    zookeeper_quorum += ","

solr_jaas_file = None
hostname_lowercase = config['agentLevelParams']['hostname'].lower()

if security_enabled:
  solr_jaas_file = solr_conf + '/solr_jaas.conf'
  solr_kerberos_keytab = default('/configurations/solr-env/solr_kerberos_keytab', None)
  if not solr_kerberos_keytab: #Maybe against older configurations during a downgrade operation. Look for the old property
    solr_keytab=config['configurations']['solr-site']['solr.hdfs.security.kerberos.keytabfile']
    solr_kerberos_keytab = solr_keytab

  solr_kerberos_principal = default('/configurations/solr-env/solr_kerberos_principal', None)
  if solr_kerberos_principal:
    solr_kerberos_principal = solr_kerberos_principal.replace('_HOST', hostname_lowercase)
  else: #Maybe against older configurations during a downgrade operation. Look for the old property
    solr_site = dict(config['configurations']['solr-site'])
    solr_principal = solr_site['solr.hdfs.security.kerberos.principal']
    solr_principal = solr_principal.replace('_HOST', hostname_lowercase)
    solr_site['solr.hdfs.security.kerberos.principal']=solr_principal
    solr_kerberos_principal = solr_principal

  solr_web_kerberos_keytab = config['configurations']['solr-env']['solr_web_kerberos_keytab']
  solr_web_kerberos_principal = default('/configurations/solr-env/solr_web_kerberos_principal', None)
  if solr_web_kerberos_principal:
    solr_web_kerberos_principal = solr_web_kerberos_principal.replace('_HOST', hostname_lowercase)
  solr_kerberos_name_rules = config['configurations']['solr-env']['solr_kerberos_name_rules']

solr_xml_content = default('configurations/solr-xml/content', None)
solr_log4j_content = default('configurations/solr-log4j/content', None)

solr_client_custom_log4j = "solr-client-log4j" in config['configurations']

restart_during_downgrade = False
upgrade_direction = default("/commandParams/upgrade_direction", None)
restart_during_downgrade = (upgrade_direction == Direction.DOWNGRADE)



smokeuser = config['configurations']['cluster-env']['smokeuser']
smoke_user_keytab = config['configurations']['cluster-env']['smokeuser_keytab']
smokeuser_principal = config['configurations']['cluster-env']['smokeuser_principal_name']

hadoop_conf_dir = conf_select.get_hadoop_conf_dir()
hadoop_bin_dir = stack_select.get_hadoop_dir("bin")
hdfs_user = config['configurations']['hadoop-env']['hdfs_user']
hdfs_site = config['configurations']['hdfs-site']
default_fs = config['configurations']['core-site']['fs.defaultFS']
dfs_type = default("/clusterLevelParams/dfs_type", "")
hdfs_user_keytab = config['configurations']['hadoop-env']['hdfs_user_keytab']
hdfs_principal_name = config['configurations']['hadoop-env']['hdfs_principal_name']
kinit_path_local = status_params.kinit_path_local

solr_wait_for_live_nodes = int(default("/configurations/solr-env/solr_wait_for_live_nodes", 0))
solr_delete_write_locks = default("/configurations/solr-env/solr_delete_write_locks", True)

zkcli_extra = format("source {solr_conf}/solr-env.sh; ")
if security_enabled:
  solr_kinit_cmd = format("{kinit_path_local} -kt {solr_kerberos_keytab} {solr_kerberos_principal}; ")
  solr_zk_creds_and_acls = "export SOLR_ZK_CREDS_AND_ACLS=${SOLR_AUTHENTICATION_OPTS}; "

  zookeeper_principal_name = default("/configurations/zookeeper-env/zookeeper_principal_name", "zookeeper/_HOST@EXAMPLE.COM")
  #external_zk_principal_enabled = default("/configurations/solr-env/cloudera_hdp_solr_zookeeper_external_enabled", False)
  #external_zk_principal_name = default("/configurations/solr-env/cloudera_hdp_solr_zookeeper_external_principal", "zookeeper/_HOST@EXAMPLE.COM")

  zk_principal_name = zookeeper_principal_name # TODO: set this differently if external zookeeper is enabled
  zk_principal_user = zk_principal_name.split('/')[0]
  zkcli_jvm_flags = format('ZKCLI_JVM_FLAGS="-Dzookeeper.sasl.client=true -Dzookeeper.sasl.client.username={zk_principal_user} -Dzookeeper.sasl.clientconfig=Client"')

  zkcli_extra+=solr_kinit_cmd
  zkcli_extra+=solr_zk_creds_and_acls
  zkcli_extra+=zkcli_jvm_flags

zkcli_root_prefix = format("{zkcli_extra} JAVA_HOME={java64_home} {solr_dir}/server/scripts/cloud-scripts/zkcli.sh -zkhost {zookeeper_quorum} -cmd ")
zkcli_prefix = format("{zkcli_extra} JAVA_HOME={java64_home} {solr_dir}/server/scripts/cloud-scripts/zkcli.sh -zkhost {zookeeper_quorum}{solr_znode} -cmd ")

ams_collector_hosts = ",".join(default("/clusterHostInfo/metrics_collector_hosts", []))
metrics_enabled = ams_collector_hosts != ''
if metrics_enabled:
  metrics_http_policy = config['configurations']['ams-site']['timeline.metrics.service.http.policy']
  ams_collector_protocol = 'http'
  if metrics_http_policy == 'HTTPS_ONLY':
    ams_collector_protocol = 'https'
  ams_collector_port = str(get_port_from_url(config['configurations']['ams-site']['timeline.metrics.service.webapp.address']))
else:
  ams_collector_port = ''
  ams_collector_protocol = ''


import functools
from resource_management.libraries.functions.get_not_managed_resources import get_not_managed_resources
#create partial functions with common arguments for every HdfsDirectory call
#to create hdfs directory we need to call params.HdfsDirectory in code
HdfsResource = functools.partial(
  HdfsResource,
  user=hdfs_user,
  hdfs_resource_ignore_file = "/var/lib/ambari-agent/data/.hdfs_resource_ignore",
  security_enabled = security_enabled,
  keytab = hdfs_user_keytab,
  kinit_path_local = kinit_path_local,
  hadoop_bin_dir = hadoop_bin_dir,
  hadoop_conf_dir = hadoop_conf_dir,
  principal_name = hdfs_principal_name,
  hdfs_site = hdfs_site,
  default_fs = default_fs,
  immutable_paths = get_not_managed_resources(),
  dfs_type = dfs_type
)
